initializer <- function(n) {
    # Seed for initial data generation
    set.seed(100)
    dqrng::dqset.seed(100)
    widths <- c(2,1,2,2,6,1,3,8,8,1,1,1,1,2,1,3,1,1,4,1,3,2,1,3,1,3,2,1,1,1,1,1,1,1,1,1,2,1,
                1,3,3,2,3,3,2,1,2,2,4,2,3,3,8,3,3,8,3,4,2,3,4,4,4,2,1,2,1,1,1,1,1,4,4,4,4)
    colnames <- c("form","versn","centre","runit","serial","numsur","samunit","dexam","mbirth",
                  "agegr","sex","marit","edlevel","school","cigs","numcigs","daycigs","evercig",
                  "stop","iflyear","maxcigs","cigage","cigarsm","cigar","pipesm","pipe","othersm",
                  "hibp","drugs","bprecd","hich","chdt","chrx","chrecd","asp","menop","agem",
                  "horm","pill","syst1","diast1","rz1","syst2","diast2","rz2","cuff","arm",
                  "bpcoder","timebp","rtemp","chol","choldl","dchol","hdl","hdldl","dhdl","scn",
                  "cotin","carbmon","height","weight","waist","hip","whcoder","oversion","eage",
                  "eageg","cohort1","cohort2","edtert1","edtert2","systm","diastm","chola","hdla")
    monica <- read.fwf("data/form04_3.txt", widths = widths, col.names = colnames)

    age_structure <- read.csv2(file = "data/vaesto_3112_2017.csv", header = TRUE)
    sex <- rbin(n)
    mf <- age_structure[,2:3] 
    age_structure[,2:3] <- mf / colSums(mf)[col(mf)]
    age <- (sex == 0L) * sample(30:99, n, replace = TRUE, prob = age_structure[,2]) +
           (sex == 1L) * sample(30:99, n, replace = TRUE, prob = age_structure[,3])
    smoking <- sex_age_init(n, sex, age, 
                            c(0.319, 0.298, 0.278, 0.219, 0.109, 0.016),
                            c(0.229, 0.193, 0.184, 0.147, 0.054, 0.016), 30, 80, 10)

    parent_stroke <- rbin(n, prob = 0.127)
    high_glucose <- sex_age_init(n, sex, age, 
                                 c(0.013, 0.043, 0.080, 0.114, 0.128, 0.110), 
                                 c(0.019, 0.012, 0.030, 0.055, 0.121, 0.093), 30, 80, 10)
    stroke <- sex_age_init(n, sex, age, 
                           c(0.027, 0.063, 0.091, 0.163), 
                           c(0.013, 0.029, 0.061, 0.137), 50, 80, 10)
    age_at_stroke <- rep(0.0, n)
    age_at_stroke[stroke == 1L] <- pmax(age[stroke == 1L] - 
        (rweibull(sum(stroke), shape = 0.509246, scale = 1634.480968)) / 365.0, 30)

    # decoding missing values
    monica$height[monica$height == 999] <- NA
    monica$weight[monica$weight == 9999] <- NA
    monica$waist[monica$waist == 9999] <- NA
    monica$systm[monica$systm == 9999] <- NA
    monica$chol[monica$chol >= 888] <- NA
    monica$hdl[monica$hdl >= 777] <- NA

    # unit transformations
    monica$weight <- monica$weight / 10 # transform to kg
    monica$waist <- monica$waist / 10 # transform to cm
    monica$systm <- monica$systm / 10 # transform to mmHg
    monica$chol <- monica$chol / 10 # transform to mmol/l
    monica$hdl <- monica$hdl / 100 # transform to mmol/l
    monica$bmi <- monica$weight / (monica$height / 100)^2
    monica$sex <- monica$sex - 1 # female = 1
    #monica$smoking <- 0 + (monica$cigs %in% c(1,3))
    monica$smoking <- 0 + (monica$cigs %in% c(1, 3) | 
                           monica$cigarsm %in% c(1, 3) | 
                           monica$pipesm %in% c(1, 3))

    selcol <- c("bmi", "waist", "chol", "hdl", "systm")
    m <- length(selcol)
    monica2 <- monica[,c("eage", "sex", "smoking", selcol)]
    out <- c(which.min(monica2$waist), which.max(monica2$chol))
    monica2 <- monica2[-out,]
    monica2$agegr <- cut(monica2$eage, breaks = c(0, 30, 40, 50, 60, 70), right = FALSE)

    # Generate initial values in each subgroup of sex, smoking and age
    norm_grp <- expand.grid(sex = 0:1, smoking = 0:1, agegr = 1:5)
    k <- nrow(norm_grp)
    norm_dat <- vector(mode = "list", length = k)
    norm_par <- vector(mode = "list", length = k)

    for (i in 1:k) {
        ind <- (monica2$sex == norm_grp[i,1] & 
                monica2$smoking == norm_grp[i,2] & 
                as.numeric(monica2$agegr) == norm_grp[i,3])
        ind[is.na(ind)] <- FALSE
        norm_par[[i]]$trans <- list()
        norm_dat[[i]] <- matrix(0, sum(ind), m)
        for (j in 1:m) {
            bc <- bestNormalize::boxcox(monica2[ind,selcol[j]])
            norm_par[[i]]$trans[[j]] <- bc
            norm_dat[[i]][,j] <- bc$x.t
        }
        norm_par[[i]]$mu <- colMeans(norm_dat[[i]], na.rm = TRUE)
        norm_par[[i]]$Sigma <- cov(norm_dat[[i]], use = "complete.obs")
    }

    X <- matrix(0, n, m)

    age_mon <- as.numeric(cut(age, breaks = c(0, 30, 40, 50, 60, 100), right = FALSE))
    for (i in 5:k) {
        ind <- (sex == norm_grp[i,1] &
                smoking == norm_grp[i,2] & 
                age_mon == norm_grp[i,3])
        temp <- MASS::mvrnorm(sum(ind), mu = norm_par[[i]]$mu, Sigma = norm_par[[i]]$Sigma)
        for (j in 1:m) {
            temp[,j] <- predict(norm_par[[i]]$trans[[j]], temp[,j], inverse = TRUE)
        }
        X[ind,] <- temp
    }

    # Limits data to values that have been actually observed
    X <- impose_limits(X, sex, smoking, age_mon, 
                       list(c(16, 72.5), c(50, 152), c(2.5, 11.5), c(0.37, 3.62), c(78, 252)))

    bmi <- X[,1]
    waist_circumference <- X[,2]
    cholesterol <- X[,3]
    hdl_cholesterol <- X[,4]
    blood_pressure <- X[,5]

    # Generating diabetes status based on FINDRISC risk score
    diabetes_par <- c(-5.514, 0.628, 0.892, 0.165, 1.096, 0.857, 1.350, 2.139)
    D <- matrix(0, n, 8)
    D[,1] <- 1
    D[,2] <- (age >= 45 & age < 54)
    D[,3] <- (age >= 54)
    D[,4] <- (bmi > 25 & bmi <= 30)
    D[,5] <- (bmi > 30)
    D[,6] <- (sex == 0L) * (waist_circumference >= 94 & waist_circumference < 102) +
             (sex == 1L) * (waist_circumference >= 80 & waist_circumference < 88)
    D[,7] <- (sex == 0L) * (waist_circumference >= 102) + 
             (sex == 1L) * (waist_circumference >= 88)
    D[,8] <- high_glucose
    diabetes_risk <- expit(as.numeric(D %*% diabetes_par))
    diabetes_risk[sex == 0L] <- diabetes_risk[sex == 0L] / mean(diabetes_risk[sex == 0L]) * 0.15
    diabetes_risk[sex == 1L] <- diabetes_risk[sex == 1L] / mean(diabetes_risk[sex == 1L]) * 0.10
    diabetes <- rbern(diabetes_risk)

    return(data.table(
        alive = rep(1L, n),
        age = as.double(age),
        age_at_start = as.double(age),
        age_at_stroke = age_at_stroke,
        sex = sex,
        smoking = smoking,
        diabetes = diabetes,
        parent_stroke = parent_stroke,
        high_glucose = high_glucose,
        stroke = stroke,
        bmi = bmi,
        waist_circumference = waist_circumference,
        cholesterol = cholesterol,
        hdl_cholesterol = hdl_cholesterol,
        blood_pressure = blood_pressure,
        cause_of_death = rep(0L, n),
        id = 1:n
    ))

}
